*+====================================================================
*+
*+    Source Module => F:\DBF_Y2K\DATEFUNC.PRG
*+
*+    A Year 2000 date scanning project by Phil Barnett and Ross McKenzie
*+    
*+    DBF_Y2K is Public Domain and complete Source Code is provided.
*+
*+    Functions: Function D2C()
*+               Function C2D()
*+               Function stod()
*+               Function gdateformt()
*+               Procedure sdateformt()
*+               Function ischardate()
*+               Function doy2date()
*+               Function isleap()
*+               Function NiceDate()
*+
*+    Reformatted by Click! 1.12 on Dec-1-1997 at 11:42 pm
*+
*+====================================================================

#include "COMMON.CH"
#include "SET.CH"

static cFullPath  := '|||'
static lFiveDigit
static lSixDigit

*+--------------------------------------------------------------------
*+
*+    Function D2C()
*+
*+--------------------------------------------------------------------
*+
function D2C( DDATE, CMASK )

CMASK := upper( CMASK )

if at( 'YYYY', CMASK ) > 0
   CMASK := strtran( CMASK, 'YYYY', str( year( DDATE ), 4 ) )
else
   CMASK := strtran( CMASK, 'YY', right( str( year( DDATE ), 4 ), 2 ) )
endif

CMASK := strtran( CMASK, 'MM', strzero( month( DDATE ), 2 ) )
CMASK := strtran( CMASK, 'DD', strzero( day( DDATE ), 2 ) )

return CMASK

*+--------------------------------------------------------------------
*+
*+    Function C2D()
*+
*+    Called from ( datefunc.prg )   6 - function ischardate()
*+
*+--------------------------------------------------------------------
*+
function C2D( CDATE, CMASK )

local dRetVal
local aDateFormat := gdateformt()

CMASK := upper( CMASK )

set( _SET_DATEFORMAT, "yyyy.mm.dd" )    // ANSI

if at( 'YYYY', CMASK ) > 0
   __SetCentury( .T. )
   dRetVal := ctod( substr( CDATE, at( 'YYYY', CMASK ), 4 ) + '.' + ;
                    substr( CDATE, at( 'MM', CMASK ), 2 ) + '.' + ;
                    substr( CDATE, at( 'DD', CMASK ), 2 ) )
else
   __SetCentury( .F. )
   dRetVal := ctod( substr( CDATE, at( 'YY', CMASK ), 2 ) + '.' + ;
                    substr( CDATE, at( 'MM', CMASK ), 2 ) + '.' + ;
                    substr( CDATE, at( 'DD', CMASK ), 2 ) )
endif

sdateformt( aDateFormat )

return dRetVal

#ifndef CT3

*+--------------------------------------------------------------------
*+
*+    Function stod()
*+
*+    Called from ( datefunc.prg )   1 - function doy2date()
*+
*+--------------------------------------------------------------------
*+
function stod( cDate )

local dRetVal     := ctod( '' )
local aDateFormat := gdateformt()

__SetCentury( .T. )
set( _SET_DATEFORMAT, "yyyy.mm.dd" )    // ANSI

cDate := alltrim( cDate )

if len( cDate ) == 8

   dRetVal := ctod( left( cDate, 4 ) + '.' + ;
                    substr( cDate, 5, 2 ) + '.' + ;
                    substr( cDate, 7, 2 ) )

endif

sdateformt( aDateFormat )

return dRetVal

#endif

*+--------------------------------------------------------------------
*+
*+    Function gdateformt()
*+
*+    Called from ( datefunc.prg )   1 - function c2d()
*+                                   1 - function stod()
*+
*+--------------------------------------------------------------------
*+
function gdateformt()

return { __SetCentury(), set( _SET_DATEFORMAT ), set( _SET_EPOCH ) }

*+--------------------------------------------------------------------
*+
*+    Procedure sdateformt()
*+
*+    Called from ( datefunc.prg )   1 - function c2d()
*+                                   1 - function stod()
*+
*+--------------------------------------------------------------------
*+
procedure sdateformt( aDateSet )

__SetCentury( aDateSet[ 1 ] )
set( _SET_DATEFORMAT, aDateSet[ 2 ] )
set( _SET_EPOCH, aDateSet[ 3 ] )

return

*+--------------------------------------------------------------------
*+
*+    Function ischardate()
*+
*+    Called from ( dbf_y2k.prg  )   2 - procedure dbf_y2k()
*+
*+--------------------------------------------------------------------
*+
function ischardate( cData, dDate )

// Returns
// 0 = empty field
// 1 = valid char date in cData
// 2 = cData contains no date info

local nDataLen
local nRetVal  := iif( empty( cData ), 0, 2 )

if cFullPath == '|||'
   cFullPath  := if( file( 'DBF_Y2K.INI' ), 'DBF_Y2K', RootName( ft_origin() ) )
   lFiveDigit := ProfileString( cFullPath + '.INI', 'DBF_Y2K', 'FIVE_DIGIT_DATES', 'YES' ) == 'YES'
   lSixDigit  := ProfileString( cFullPath + '.INI', 'DBF_Y2K', 'SIX_DIGIT_DATES', 'YES' ) == 'YES'
endif

// treat empty date statement like empty cData
if '  /  /  ' $ cData .or. '  .  .  ' $ cData
   nRetVal := 0
endif

begin sequence

   if nRetVal > 0

      // just retain digits
      cData := charonly( '0123456789', cData )

      nDataLen := len( cData )

      if lFiveDigit .and. nDataLen == 5
         dDate := doy2date( cData, 'YYDDD' )
         if !empty( dDate )
            nRetVal := 1
            break
         endif
         dDate := doy2date( cData, 'DDDYY' )
         if !empty( dDate )
            nRetVal := 1
            break
         endif
      endif

      if lSixDigit .and. nDataLen == 6
         dDate := c2d( cData, 'YYMMDD' )
         if !empty( dDate )
            nRetVal := 1
            break
         endif
         dDate := c2d( cData, 'DDMMYY' )
         if !empty( dDate )
            nRetVal := 1
            break
         endif
         dDate := c2d( cData, 'MMDDYY' )
         if !empty( dDate )
            nRetVal := 1
            break
         endif
         dDate := c2d( cData, 'YYDDMM' )
         if !empty( dDate )
            nRetVal := 1
            break
         endif
         dDate := c2d( cData, 'DDYYMM' )
         if !empty( dDate )             // not likely
            nRetVal := 1
            break
         endif
         dDate := c2d( cData, 'MMYYDD' )
         if !empty( dDate )             // not likely
            nRetVal := 1
            break
         endif
      endif

   endif

end sequence

return nRetVal

*+--------------------------------------------------------------------
*+
*+    Function doy2date()
*+
*+    Called from ( datefunc.prg )   2 - function ischardate()
*+
*+--------------------------------------------------------------------
*+
function doy2date( cData, cMask )

local dDate    := ctod( '' )
local YEARDAYS
local m
local nDoy
local nYear
local nMonth
local nDay

begin sequence

   default cMask to ''

   cMask := upper( cMask )

   cData := charonly( '0123456789', cData )

   do case
   case cMask == 'YYDDD'
      nYear := val( left( cData, 2 ) ) + 1900
      nDoy  := val( right( cData, 3 ) )
   case cMask == 'DDDYY'
      nYear := val( right( cData, 2 ) ) + 1900
      nDoy  := val( left( cData, 3 ) )
   otherwise
      break
   endcase

   if nDoy < 1
      break
   endif

   if isleap( nYear )
      YEARDAYS := { 0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335 }
   else
      YEARDAYS := { 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334 }
   endif

   for m := 12 to 1 step - 1
      if nDoy > yeardays[ m ]
         nMonth := m
         nDay   := nDoy - yeardays[ m ]
         exit
      endif
   next

   dDate := stod( str( nYear, 4 ) + strzero( nMonth, 2 ) + strzero( nDay, 2 ) )

end sequence

return dDate

*+--------------------------------------------------------------------
*+
*+    Function isleap()
*+
*+    Called from ( datefunc.prg )   1 - function doy2date()
*+
*+--------------------------------------------------------------------
*+
function isleap( nYear )

return ( ( nYear % 4 == 0 ) .and. !( nYear % 100 == 0 ) ) .or. ( nYear % 400 == 0 )

*+--------------------------------------------------------------------
*+
*+    Function NiceDate()
*+
*+    Called from ( dbf_y2k.prg  )   1 - procedure dbf_y2k()
*+
*+--------------------------------------------------------------------
*+
function NiceDate( dDate )

local cDateString := "<< Invalid Date >>"

// this is a nice unambiguous date display format for humans.

if valtype( dDate ) == 'D'
   cDateString := left( cmonth( dDate ), 3 ) + '-' + ;
                        ltrim( str( day( dDate ) ) ) + '-' + ;
                        str( year( dDate ), 4 )
endif

return cDateString

*+ EOF: DATEFUNC.PRG
