*+====================================================================
*+
*+    Source Module => F:\DBF_Y2K\DBF_Y2K.PRG
*+
*+    A Year 2000 date scanning project by Phil Barnett and Ross McKenzie
*+    
*+    DBF_Y2K is Public Domain and complete Source Code is provided.
*+
*+    Functions: Procedure dbf_y2k()
*+               Procedure Info()
*+               Procedure MyErrorHandler()
*+               Procedure aadduniq()
*+
*+       Tables: use ( cThisDir + aFiles[ x, 1 ] ) new SHARED readonly alias testfile
*+               use ( cThisDir + aFiles[ x, 1 ] ) new exclusive readonly alias testfile
*+
*+    Reformatted by Click! 1.12 on Dec-1-1997 at 11:42 pm
*+
*+====================================================================

#include "COMMON.CH"
#include "DBSTRUCT.CH"
#include "BOX.CH"
#include "INKEY.CH"

#translate c( < foo > ) = > max( 40 - int( len( < foo > ) / 2 ), 0 )

/*

DBF_Y2K.EXE

Written by Phil Barnett
MDX and CDX detection by Ross McKenzie  (added for version 1.4)
New screen & recursive operation by Ross McKenzie (added for version 1.5)

New error handler        \
New .INI file defaults    > by Phil Barnett ( 1.6 )
New processing abilities /

Fix bug in C2D() function.                 \  by Phil ( 1.7 )
Add field name summaries at end of report  /

*/

*+--------------------------------------------------------------------
*+
*+    Procedure dbf_y2k()
*+
*+--------------------------------------------------------------------
*+
procedure dbf_y2k

local getlist       := {}
local cTopLine      := 'Year 2000 .DBF Scanner 1.7'
local cBotLine      := " F1 Info " + chr( 179 ) + "       A collaboration by Phil Barnett and Ross McKenzie       " + chr( 179 ) + " 1997 "
local cColour
local aDates        := {}
local afields       := {}
local nfields
local aFiles
local aNumFiles
local i
local x
local y
local z
local dMHigh        := ctod( '' )
local dMLow         := ctod( '' )
local dFHigh
local dFLow
local crlf          := chr( 13 ) + chr( 10 )
local nHandle
local lGotDates     := .t.
local nDbfHdl
local checkbyte
local cIniFile      := rootname( ft_origin() ) + '.INI'
local aIni          := {}
local nIni
local aExc          := {}
local nExc
local cThisLine
local lGotOne
local aAllEntries   := {}               //result of directory(cThisDir)
local n             //for loop pointer
local nAllEntries
local aSubDirs      := {}               //all sub directory names
local nSubDirs      := 0                //qty of sub directories
local nChecked      := 0                //qty of directories checked
local aDBFDirs      := {}               //array of .DBF directories
local nDBFChkd      := 0                //qty .DBF files processed
local bErrorHandler
local bLastHandler
local objError
local NullDate      := ctod( '' )
local dDate         := NullDate
local aUsed
local aHigh
local aLow
local uThisField
local nResult
local oBufObj       := bInit( cIniFile )
local cFullPath     := if( file( 'DBF_Y2K.INI' ), 'DBF_Y2K', RootName( ft_origin() ) )
local cOpenMode     := ProfileString( cFullPath + '.INI', 'DBF_Y2K', 'OPENMODE', 'SHARED' )
local cOutFile      := pad( ProfileString( cFullPath + '.INI', 'DBF_Y2K', 'OUTPUTFILE', '.\DBF_SCAN.Y2K' ), 100 )
local cThisDir      := ProfileString( cFullPath + '.INI', 'DBF_Y2K', 'STARTDIR', '.\' )
local lRecurse      := ProfileString( cFullPath + '.INI', 'DBF_Y2K', 'SUBDIRECTORY', 'NO' ) == 'YES'
local lNumeric      := ProfileString( cFullPath + '.INI', 'DBF_Y2K', 'SCAN_IN_NUMERICS', 'YES' ) == 'YES'
local lChar         := ProfileString( cFullPath + '.INI', 'DBF_Y2K', 'SCAN_IN_CHARACTER', 'YES' ) == 'YES'
local lDates        := ProfileString( cFullPath + '.INI', 'DBF_Y2K', 'SCAN_IN_DATES', 'YES' ) == 'YES'
local lUnused       := ProfileString( cFullPath + '.INI', 'DBF_Y2K', 'SHOW_EMPTY', 'YES' ) == 'YES'
local lTrigger      := ProfileString( cFullPath + '.INI', 'DBF_Y2K', 'SHOW_TRIGGER', 'YES' ) == 'YES'
local aDateFlds     := {}
local nDateFldLen
local aNumFlds      := {}
local nNumFldLen
local aCharFlds     := {}
local nCharFldLen
local aTrigFlds     := {}
local nTrigFldLen

bErrorHandler := { | objError | MyErrorHandler( objError ) }

begin sequence

   if bOpen( oBufObj )
      // read up to the [INCLUDE] section
      do while !bEof( oBufObj )
         cThisLine := upper( alltrim( bReadLine( oBufObj ) ) )
         if '[INCLUDE]' $ cThisline
            exit
         endif
      enddo
      // read in the [INCLUDE] section
      do while !bEof( oBufObj )
         cThisLine := upper( alltrim( bReadLine( oBufObj ) ) )
         if '[EXCLUDE]' $ cThisline
            exit
         endif
         if !empty( cThisLine ) .and. left( cThisLine, 2 ) <> '//'
            aadd( aIni, cThisLine )
         endif
      enddo
      // read in the [EXCLUDE] section
      do while !bEof( oBufObj )
         cThisLine := upper( alltrim( bReadLine( oBufObj ) ) )
         if !empty( cThisLine ) .and. left( cThisLine, 2 ) <> '//'
            aadd( aEXC, cThisLine )
         endif
      enddo
      bClose( oBufObj )
   else
      ?
      ? 'Could not open ' + cIniFile
      ?
      quit
   endif

   nIni := len( aIni )
   nEXC := len( aEXC )

   set century on
   set scoreboard off
   set delete on

   /*
   allow popup information about the program
   */
   set key K_F1 to info

   // make the screen look nice

   setcolor( 'W+/B' )
   clear screen
   @  1,  0 say replicate( '', 80 )         

   @  3,  7 say 'Start Directory                                Subdirectories?'         

   @  5,  7 say 'Write Report Findings to'         

   @  7,  8 say 'Ŀ'         
   @  8,  8 say '        Directories          DBFs         Dates Found     '         
   @  9,  8 say 'Ĵ'         
   @ 10,  8 say '  Candidates  Examined    Processed   Earliest   Latest '         
   @ 11,  8 say '                                                        '         
   @ 12,  8 say ''         

   cColour := setcolor( "W+/BG+" )
   scroll( 0, 0, 0, 79 )
   @  0, c( cTopLine ) say cTopLine         
   scroll( 23, 0, 0, 79 )
   @ 23, c( cBotLine ) say cBotLine         

   setcolor( cColour )

   // read the operator directives defaulting to the current directory
   // and DBFSCAN.Y2K

   if empty( cThisDir )
      cThisDir := '.\'
   endif
   cThisDir := padr( cThisDir, 100 )

   set confirm on

   @  3, 23 get cThisDir picture "@K!@S30"        
   @  3, 70 get lRecurse picture 'Y'              
   @  5, 32 get cOutFile picture "@K!S39"         
   read

   set confirm off
   setcursor( 0 )
   set key K_F1 to

   if lastkey() == 27
      break
   endif

   cThisDir := alltrim( cThisDir )

   do case
   case substr( cThisDir, 2, 1 ) == ':'                     // drive letter
      cThisDir += "\"
   case trim( cThisDir ) == '.\'        // current directory
      cThisDir := '\' + curdir() + '\'
   case left( cThisDir, 2 ) == '.\'
      cThisDir := '\' + alltrim( substr( cThisDir, 3 ) ) + '\'
   case empty( cThisDir ) .or. cThisDir == "\"
      cThisDir := "\"
   otherwise
      cThisDir := "\" + cThisDir + "\"
   endcase

   cThisDir := strtran( cThisDir, "\\", "\" )

   if empty( cOutFile )
      cOutFile := "DBF_SCAN.Y2K"
   endif

   // create the output file
   nHandle := fcreate( alltrim( cOutFile ), 0 )

   fwrite( nHandle, crlf + '     Y2K_DBF: Year 2000 Date Impact Analysis Engine' + crlf )
   fwrite( nHandle, '     Y2K Min/Max date analysis performed on ' + nicedate( date() ) + crlf )

   // walk the selected branch of the hard disk
   // recording only those with .dbf files in them

   Attention( "Examining Directories", 20, setcolor() )

   do while .t.
      // there is always at least this directory to examine
      @ 11, 12 say padc( nSubDirs + 1, 8 )         

      // zero for each subdirectory
      asize( aAllEntries, 0 )

      aAllEntries := directory( cThisdir + "*.*", "D,H" )

      nAllEntries := len( aAllEntries )

      if nAllEntries > 2
         //not an empty directory
         if file( cThisDir + "*.DBF" )
            // there are candidate files in this directory
            // add the directory's name to the list
            aadd( aDBFDirs, cThisDir )
         endif

         if lRecurse
            for n := 1 to nAllEntries

               if "D" $ aAllEntries[ n, 5 ] ;
                          .and. !( aAllEntries[ n, 1 ] == "." .or. aAllEntries[ n, 1 ] == ".." )

                  // a new candidate subdirectory found in this directory
                  // add its full path name to the list
                  aadd( aSubDirs, cThisDir + aAllEntries[ n, 1 ] + "\" )

                  // bump the subdirectory counter by 1
                  ++ nSubDirs

                  // on screen counter & information
                  @ 11, 12 say padc( nSubDirs + 1, 8 )         
                  scroll( 21, 0, 21, 79 )
                  attention( cThisDir, 21, setcolor() )
               endif
            next
         endif
      endif

      // more on screen info
      @ 11, 25 say padc( nChecked + 1, 8 )         

      if ++ nChecked > nSubDirs .or. !lRecurse
         // have examined every subdirectory
         exit
      else
         // point at the next directory to check
         cThisDir := aSubDirs[ nChecked ]
      endif
   enddo

   // tidy the screen
   scroll( 20, 0, 21, 79 )

   // sort and then loop through the candidate subdirectories
   aDBFDirs := asort( aDBFDirs )

   for n := 1 to len( aDBFDirs )
      cThisDir := aDBFDirs[ n ]

      // some more feedback
      Attention( "Processing Directory", 20, setcolor() )
      scroll( 21, 0, 21, maxcol() )
      Attention( cThisDir, 21, setcolor() )

      // record which directory is being used
      fwrite( nHandle, crlf + "Processing directory " + cThisDir + crlf )

      aFiles    := directory( cThisDir + "*.DBF" )
      aNumFiles := len( aFiles )

      // sort the files in this directory
      aFiles := asort( aFiles,,, { | a, b | a[ 1 ] < b[ 1 ] } )

      // loop through all the matching files.
      for x := 1 to aNumFiles
         ++ nDBFChkd
         @ 11, 38 say padc( nDBFChkd, 8 )         

         Attention( alltrim( str( x ) ) + ' of ' + alltrim( str( aNumFiles ) ), 13, "N/W" )

         // Determine if this file is *REALLY* a .DBF?
         nDbfHdl := fopen( cThisDir + aFiles[ x, 1 ], 0 )
         fseek( nDbfHdl, 0, 0 )
         checkbyte := ' '
         fread( nDbfHdl, @checkbyte, 1 )
         fclose( nDbfHdl )

         checkbyte := asc( checkbyte )

         if !( strzero( checkbyte, 3 ) $ '131|003|139|245' )
            fwrite( nHandle, '     ' + pad( aFiles[ x, 1 ], 20 ) + 'Checkbyte mismatch. Not a .DBF file' + crlf )
            loop
         endif

         // default database driver
         Request DBFNTX
         rddsetdefault( "DBFNTX" )

         if checkbyte == 131 .and. !file( rootname( cThisDir + aFiles[ x, 1 ] ) + '.DBT' )
            fwrite( nHandle, '     ' + pad( aFiles[ x, 1 ], 20 ) + "is missing it's .DBT file" + crlf )
            loop
         endif

         if checkbyte == 139
            if file( rootname( cThisDir + aFiles[ x, 1 ] ) + '.DBT' )
               Request DBFMDX
               rddsetdefault( "DBFMDX" )
            else
               fwrite( nHandle, '     ' + pad( aFiles[ x, 1 ], 20 ) + "is missing it's .DBT file" + crlf )
               loop
            endif
         endif

         if checkbyte == 245
            if file( rootname( cThisDir + aFiles[ x, 1 ] ) + '.FPT' )
               Request DBFCDX
               rddsetdefault( "DBFCDX" )
            else
               fwrite( nHandle, '     ' + pad( aFiles[ x, 1 ], 20 ) + "is missing it's .FPT file" + crlf )
               loop
            endif
         endif

         scroll( 22, 0, 22, maxcol() )
         Attention( aFiles[ x, 1 ], 22, setcolor() )

         bLastHandler := errorblock( bErrorHandler )

         begin sequence

            // if you got this far, it's a .DBF and we can open it.
            if cOpenMode == 'SHARED'
               use ( cThisDir + aFiles[ x, 1 ] ) new SHARED readonly alias testfile
            else
               use ( cThisDir + aFiles[ x, 1 ] ) new exclusive readonly alias testfile
            endif

         recover using objError

            if used()
               testfile->( dbclosearea() )
            endif

            fwrite( nHandle, '     ' + pad( aFiles[ x, 1 ], 20 ) + 'Possibly corrupted. Unable to Open.' + crlf )

            pop_msg( { 'Unable to open data file.', ;
                       '', ;
                       'Possible corrupted .DBF file', ;
                       '', ;
                       cThisDir + trim( aFiles[ x, 1 ] ) },, 'G+/R', 'W+/R' )

            loop

         end sequence

         errorblock( bLastHandler )

         if neterr()
            // close the file
            if used()
               testfile->( dbclosearea() )
            endif
            fwrite( nHandle, '     ' + pad( aFiles[ x, 1 ], 20 ) + 'Unable to Open' + crlf )
            loop
         endif

         // create some data gathering arrays
         aFields := testfile->( dbstruct() )
         nFields := len( aFields )

         aDates := array( nFields )
         afill( aDates, .t. )

         aUsed := array( nFields )
         afill( aUsed, .f. )

         aHigh := array( nFields )
         afill( aHigh, NullDate )

         aLow := aclone( aHigh )

         if testfile->( reccount() ) > 0

            // put a progress meter on the screen
            init_progress( 17, testfile->( lastrec() ), '' )

            // loop through the open file
            do while testfile->( !eof() )

               // increment the progress display
               progress()

               // scan the fields
               for y := 1 to nFields

                  if !aUsed[ y ]
                     aUsed[ y ] := !empty( testfile->( fieldget( y ) ) )
                  endif

                  if aUsed[ y ] .and. aDates[ y ]

                     uThisField := testfile->( fieldget( y ) )

                     do case
                     case aFields[ y, DBS_TYPE ] == 'D' .and. lDates

                        if !empty( uThisField )

                           if aLow[ y ] == NullDate
                              aLow[ y ] := uThisfield
                           else
                              aLow[ y ] := min( aLow[ y ], uThisField )
                           endif

                           if aHigh[ y ] == NullDate
                              aHigh[ y ] := uThisField
                           else
                              aHigh[ y ] := max( aHigh[ y ], uThisField )
                           endif

                        endif

                     case aFields[ y, DBS_TYPE ] == 'C' .and. lChar

                        nResult := ischardate( uThisField, @dDate )

                        if nResult == 1
                           if aLow[ y ] == NullDate
                              aLow[ y ] := dDate
                           else
                              aLow[ y ] := min( aLow[ y ], dDate )
                           endif

                           if aHigh[ y ] == NullDate
                              aHigh[ y ] := dDate
                           else
                              aHigh[ y ] := max( aHigh[ y ], dDate )
                           endif
                        endif

                        if nResult == 2
                           // eliminate this field from the date scan
                           aDates[ y ] := .f.
                           aLow[ y ]   := NullDate
                           aHigh[ y ]  := NullDate
                        endif

                     case aFields[ y, DBS_TYPE ] == 'N' .and. lNumeric

                        nResult := ischardate( str( uThisField ), @dDate )

                        if nResult == 1
                           if aLow[ y ] == NullDate
                              aLow[ y ] := dDate
                           else
                              aLow[ y ] := min( aLow[ y ], dDate )
                           endif

                           if aHigh[ y ] == NullDate
                              aHigh[ y ] := dDate
                           else
                              aHigh[ y ] := max( aHigh[ y ], dDate )
                           endif

                        endif

                        if nResult == 2
                           // eliminate this field from the date scan
                           aDates[ y ] := .f.
                           aLow[ y ]   := NullDate
                           aHigh[ y ]  := NullDate
                        endif

                     otherwise

                        // eliminate this field from the date scan
                        aDates[ y ] := .f.
                        aLow[ y ]   := NullDate
                        aHigh[ y ]  := NullDate

                     endcase

                  endif

               next

               testfile->( dbskip( 1 ) )

            enddo

            // remove the progress meter
            kill_progress()

            // consolidate the date min/max for this file

            dFLow  := NullDate
            dFHigh := NullDate
            for y := 1 to nFields
               if aDates[ y ] .and. aUsed[ y ]
                  if empty( dFLow )
                     dFLow := aLow[ y ]
                  else
                     dFLow := min( dFLow, aLow[ y ] )
                  endif
                  if empty( dFHigh )
                     dFhigh := aHigh[ y ]
                  else
                     dFHigh := max( dFHigh, ahigh[ y ] )
                  endif
               endif
            next

            // adjust the global low and high date collector variables
            if !empty( dFLow )
               if empty( dMLow )
                  dMLow := dFLow
               else
                  dMLow := min( dMLow, dFLow )
               endif
            endif
            if !empty( dFHigh )
               if empty( dMHigh )
                  dMHigh := dFHigh
               else
                  dMHigh := max( dMHigh, dFHigh )
               endif
            endif

            // update the screen advice
            @ 11, 52 say str( year( dMLow ), 4 )  colour "W+/B"        
            @ 11, 63 say str( year( dMHigh ), 4 ) colour "W+/B"        

            // output the findings to the report file
            if empty( dFHigh ) .and. empty( dFLow )
               fwrite( nHandle, crlf + '     ' + pad( aFiles[ x, 1 ], 20 ) + '*' + crlf )
               lGotDates := .f.
            else
               fwrite( nHandle, crlf + '     ' + pad( aFiles[ x, 1 ], 20 ) + 'Low ' + str( year( dFLow ), 4 ) + ' High ' + str( year( dFHigh ), 4 ) + crlf )
            endif

            if lUnused
               for y := 1 to nFields
                  if !aUsed[ y ]
                     fwrite( nHandle, '          Unused Field : ' + fieldname( y ) + crlf )
                  endif
               next
            endif

            for y := 1 to nFields
               if aDates[ y ] .and. aUsed[ y ]
                  lGotOne := .t.
                  for i := 1 to nEXC
                     if aExc[ i ] $ fieldname( y )
                        lGotOne := .F.
                        exit
                     endif
                  next
                  if lGotOne
                     if aFields[ y, DBS_TYPE ] == 'D'
                        fwrite( nHandle, '            Date Field : ' + pad( fieldname( y ), 11 ) )
                        aadduniq( aDateFlds, pad( fieldname( y ), 11 ) )
                     endif
                     if aFields[ y, DBS_TYPE ] == 'N'
                        fwrite( nHandle, '         Numeric Field : ' + pad( fieldname( y ), 11 ) )
                        aadduniq( aNumFlds, pad( fieldname( y ), 11 ) )
                     endif
                     if aFields[ y, DBS_TYPE ] == 'C'
                        fwrite( nHandle, '       Character Field : ' + pad( fieldname( y ), 11 ) )
                        aadduniq( aCharFlds, pad( fieldname( y ), 11 ) )
                     endif
                     fwrite( nHandle, str( year( aLow[ y ] ), 4 ) + '-' + str( year( aHigh[ y ] ), 4 ) + crlf )
                  endif
               endif
            next

         else

            if len( aDates ) > 0
               fwrite( nHandle, crlf + '     ' + pad( aFiles[ x, 1 ], 20 ) + 'Empty Table' + crlf )
            else
               fwrite( nHandle, crlf + '     ' + pad( aFiles[ x, 1 ], 20 ) + '* Empty Table' + crlf )
            endif

         endif

         if lTrigger
            for y := 1 to nFields
               for z := 1 to nIni
                  if aIni[ z ] $ aFields[ y, DBS_NAME ] .and. aFields[ y, DBS_TYPE ] <> 'D'
                     lGotOne := .t.
                     for i := 1 to nEXC
                        if aExc[ i ] $ aFields[ y, DBS_NAME ]
                           lGotOne := .F.
                           exit
                        endif
                     next
                     if lGotOne
                        fwrite( nHandle, '               Trigger : ' + pad( aIni[ z ], 11 ) + '   in Field Name: ' + aFields[ y, DBS_NAME ] + crlf )
                        aadduniq( aTrigFlds, pad( fieldname( y ), 11 ) )
                        exit
                     endif
                  endif
               next
            next
         endif

         // close the file
         testfile->( dbclosearea() )

      next

   next

   // output the master findings
   if empty( dMHigh ) .and. empty( dMLow )
      fwrite( nHandle, crlf + crlf + '     All Examined Files: *' + crlf + crlf )
   else
      fwrite( nHandle, crlf + crlf + '     All Examined Files: Low ' + str( year( dMLow ), 4 ) + ' High ' + str( year( dMHigh ), 4 ) + crlf + crlf )
   endif

   nDateFldLen := len( aDateFlds )
   if nDateFldLen > 0
      aDateFlds := asort( aDateFlds )
      fwrite( nHandle, "     Date Field Name Summary:" + crlf + crlf )
      for x := 1 to nDateFldLen
         fwrite( nHandle, '            ' + aDateFlds[ x ] + crlf )
      next
   else
      fwrite( nHandle, "     * Means no 'Date Type' fields found in file" + crlf )
   endif

   nNumFldLen := len( aNumFlds )
   if nNumFldLen > 0
      aNumFlds := asort( aNumFlds )
      fwrite( nHandle, crlf + "     Numeric Field Name Summary:" + crlf + crlf )
      for x := 1 to nNumFldLen
         fwrite( nHandle, '            ' + aNumFlds[ x ] + crlf )
      next
   endif

   nCharFldLen := len( aCharFlds )
   if nCharFldLen > 0
      aCharFlds := asort( aCharFlds )
      fwrite( nHandle, crlf + "     Character Field Name Summary:" + crlf + crlf )
      for x := 1 to nCharFldLen
         fwrite( nHandle, '            ' + aCharFlds[ x ] + crlf )
      next
   endif

   if lTrigger
      nTrigFldLen := len( aTrigFlds )
      if nTrigFldLen > 0
         aTrigFlds := asort( aTrigFlds )
         fwrite( nHandle, crlf + "     Trigger Field Name Summary:" + crlf + crlf )
         for x := 1 to nTrigFldLen
            fwrite( nHandle, '            ' + aTrigFlds[ x ] + crlf )
         next
      endif
   endif

end sequence

//tidy up the screen
scroll( 13, 0, 22, 79 )

// close the output file
fclose( nHandle )

setcursor( 1 )

setpos( 23, 0 )

quit

return

*+--------------------------------------------------------------------
*+
*+    Procedure Info()
*+
*+--------------------------------------------------------------------
*+
procedure Info

pop_msg( { "DBF_Y2K examines .DBF files to determine possible", ;
           "exposure to date related problems associated with the", ;
           "year 1999 to 2000 century indicator crossover problem.", ;
           "", ;
           "True 'date type' fields and dates which are stored in", ;
           "common character or numeric formats are located.", ;
           "", ;
           "The range of earliest and latest dates found are logged.", ;
           "", ;
           "An ascii text report is generated containing the summary", ;
           "of all findings.", ;
           "", ;
           "Field names are also tested against common date related", ;
           "usage such as 'Year' and 'Mth'. An ascii file, DBF_Y2K.INI,", ;
           "holds these trigger words. You can edit DBF_Y2K.INI as needed.", ;
           "", ;
           "DBF_Y2K processing can be limited to a single directory,", ;
           "a directory branch, or an entire partition." } )

return

*+--------------------------------------------------------------------
*+
*+    Procedure MyErrorHandler()
*+
*+    Called from ( dbf_y2k.prg  )   1 - procedure dbf_y2k()
*+
*+--------------------------------------------------------------------
*+
procedure MyErrorHandler( objError )

break objError

return

*+--------------------------------------------------------------------
*+
*+    Procedure aadduniq()
*+
*+    Called from ( dbf_y2k.prg  )   4 - procedure dbf_y2k()
*+
*+--------------------------------------------------------------------
*+
procedure aadduniq( aData, uKey )

local nPointer := ascan( aData, uKey )

if nPointer == 0
   aadd( aData, uKey )
endif

return

*+ EOF: DBF_Y2K.PRG
