*+====================================================================
*+
*+    Source Module => F:\DBF_Y2K\OBUFREAD.PRG
*+
*+    A Year 2000 date scanning project by Phil Barnett and Ross McKenzie
*+    
*+    DBF_Y2K is Public Domain and complete Source Code is provided.
*+
*+    Functions: Function BInit()
*+               Function Bopen()
*+               Function BReadLine()
*+               Function BNextLine()
*+               Static Function BDisk2Buff()
*+               Function BEof()
*+               Function BClose()
*+               Function BPosition()
*+               Function BRelative()
*+               Function BLineNumber()
*+
*+    Reformatted by Click! 1.12 on Dec-1-1997 at 11:43 pm
*+
*+====================================================================

#include "FILEIO.CH"
#include "COMMON.CH"

#define cFileName oBuffObj[ 1 ]
#define nAccMode oBuffObj[ 2 ]
#define cLineBuffer oBuffObj[ 3 ]
#define nHandle oBuffObj[ 4 ]
#define nBytesRead oBuffObj[ 5 ]
#define lFullBuff oBuffObj[ 6 ]
#define nTotBytes oBuffObj[ 7 ]
#define lIsOpen oBuffObj[ 8 ]
#define nFileBytes oBuffObj[ 9 ]
#define nFileLines oBuffObj[ 10 ]
#define cDelimiter oBuffObj[ 11 ]

/*
function test( filename )

local oBuffObj1 := binit( filename )

if bopen( oBuffObj1 )
   do while !beof( oBuffObj1 )
      ? breadline( oBuffObj1 )
   enddo
   bclose( oBuffObj1 )
else
   ? 'Cannot open ' + filename
endif

return NIL
*/

#define BUFFLEN 2560

*+--------------------------------------------------------------------
*+
*+    Function BInit()
*+
*+    Called from ( dbf_y2k.prg  )   1 - procedure dbf_y2k()
*+                ( profile.prg  )   1 - static function init_profile()
*+
*+--------------------------------------------------------------------
*+
function BInit( xFileName, xAccMode )

// Function BInit( <cFileName>, [nAccessMode] )
// Return:  oBuffObj for this file access

return { xFileName, xAccMode, '', - 1, 0, .t., 0, .t., 0, 0, chr( 13 ) + chr( 10 ) }

*+--------------------------------------------------------------------
*+
*+    Function Bopen()
*+
*+    Called from ( dbf_y2k.prg  )   1 - procedure dbf_y2k()
*+                ( profile.prg  )   1 - static function init_profile()
*+
*+--------------------------------------------------------------------
*+
function Bopen( oBuffObj )

// Return:   .t. if file was opened
// Assumes:  All file access will be done with the B* functions

default nAccMode to FO_READ             // default access mode is Read Only

nHandle := fopen( cFileName, nAccMode )

nFileBytes := fseek( nHandle, 0, FS_END )

fseek( nHandle, 0, FS_SET )

BDisk2Buff( oBuffObj )

do case
case chr( 13 ) + chr( 10 ) $ cLineBuffer
   cDelimiter := chr( 13 ) + chr( 10 )
case chr( 13 ) $ cLineBuffer
   cDelimiter := chr( 13 )
case chr( 10 ) $ cLineBuffer
   cDelimiter := chr( 10 )
endcase

return ( nHandle != - 1 )

*+--------------------------------------------------------------------
*+
*+    Function BReadLine()
*+
*+    Called from ( dbf_y2k.prg  )   3 - procedure dbf_y2k()
*+                ( profile.prg  )   1 - static function init_profile()
*+
*+--------------------------------------------------------------------
*+
function BReadLine( oBuffObj )

// Return:   The next line of the file read buffer
// Assumes:  The file pointer will be moved forward

local ThisLine
local nCrLfAt

do while .t.

   nCrLfAt := at( cDelimiter, cLineBuffer )

   if empty( nCrLfAt ) .and. lFullBuff
      BDisk2Buff( oBuffObj )
      loop
   endif

   if empty( nCrLfAt )
      ThisLine    := strtran( cLineBuffer, chr( 26 ) )
      cLineBuffer := ''
   else
      ThisLine    := left( cLineBuffer, nCrLfAt - 1 )
      cLineBuffer := substr( cLineBuffer, nCrLfAt + len( cDelimiter ) )
   endif

   exit

enddo

nFileLines ++

return ThisLine

*+--------------------------------------------------------------------
*+
*+    Function BNextLine()
*+
*+--------------------------------------------------------------------
*+
function BNextLine( oBuffObj )

// Return:   The next line of the file read buffer
// Assumes:  The file pointer will be left as last positioned

local NextLine
local nCrLfAt

default cDelimiter to chr( 13 ) + chr( 10 )

do while .t.

   nCrLfAt := at( cDelimiter, cLineBuffer )

   if empty( nCrLfAt ) .and. lFullBuff
      BDisk2Buff( oBuffObj )
      loop
   endif

   if empty( nCrLfAt )
      NextLine := strtran( cLineBuffer, chr( 26 ) )
   else
      NextLine := left( cLineBuffer, nCrLfAt - 1 )
   endif

   exit

enddo

return NextLine

*+--------------------------------------------------------------------
*+
*+    Static Function BDisk2Buff()
*+
*+    Called from ( obufread.prg )   1 - function bopen()
*+                                   1 - function breadline()
*+                                   1 - function bnextline()
*+
*+--------------------------------------------------------------------
*+
static function BDisk2Buff( oBuffObj )

// Return:   .t. if there was no read error

static cDiskBuffer := ''

if len( cDiskBuffer ) != BUFFLEN
   cDiskBuffer := space( BUFFLEN )
endif

nBytesRead := fread( nHandle, @cDiskBuffer, BUFFLEN )

nTotBytes += nBytesRead

lFullBuff := ( nBytesRead == BUFFLEN )

if lFullBuff
   cLineBuffer += cDiskBuffer
else
   cLineBuffer += left( cDiskBuffer, nBytesRead )
endif

return ferror()

*+--------------------------------------------------------------------
*+
*+    Function BEof()
*+
*+    Called from ( dbf_y2k.prg  )   3 - procedure dbf_y2k()
*+                ( profile.prg  )   1 - static function init_profile()
*+
*+--------------------------------------------------------------------
*+
function BEof( oBuffObj )

// Return:   TRUE  if End of buffered file
//           FALSE if not

return !lFullBuff .and. len( cLineBuffer ) == 0

*+--------------------------------------------------------------------
*+
*+    Function BClose()
*+
*+    Called from ( dbf_y2k.prg  )   1 - procedure dbf_y2k()
*+                ( profile.prg  )   1 - static function init_profile()
*+
*+--------------------------------------------------------------------
*+
function BClose( oBuffObj )

if lIsOpen
   fclose( nHandle )
   lIsOpen := .f.
endif

oBuffObj := nil

return ferror()

*+--------------------------------------------------------------------
*+
*+    Function BPosition()
*+
*+    Called from ( obufread.prg )   1 - function brelative()
*+
*+--------------------------------------------------------------------
*+
function BPosition( oBuffObj )

// Returns the position of virtual file pointer

return nTotBytes - len( cLineBuffer )

*+--------------------------------------------------------------------
*+
*+    Function BRelative()
*+
*+--------------------------------------------------------------------
*+
function BRelative( oBuffObj )

// Returns the percentage of file processed

return BPosition( oBuffObj ) / nFileBytes

*+--------------------------------------------------------------------
*+
*+    Function BLineNumber()
*+
*+--------------------------------------------------------------------
*+
function BLineNumber( oBuffObj )

// Returns the current line number

return nFileLines

*+ EOF: OBUFREAD.PRG
